# Python modules
import time
import md5
import mmap

# 3rd party modules
import posix_ipc

# Utils for this demo
import utils

params = utils.read_params()

# Mrs. Premise has already created the semaphore and shared memory. 
# I just need to get handles to them.
memory = posix_ipc.SharedMemory(params["SHARED_MEMORY_NAME"])
semaphore = posix_ipc.Semaphore(params["SEMAPHORE_NAME"])

# MMap the shared memory
mapfile = mmap.mmap(memory.fd, memory.size)

what_i_wrote = ""

for i in xrange(0, params["ITERATIONS"]):
    utils.say("iteration %d" % i)
    if not params["LIVE_DANGEROUSLY"]:
        # Wait for Mrs. Premise to free up the semaphore.
        utils.say("Waiting to acquire the semaphore")
        semaphore.acquire()

    s = utils.read_from_memory(mapfile)

    while s == what_i_wrote:
        if not params["LIVE_DANGEROUSLY"]:
            # Release the semaphore...
            utils.say("Releasing the semaphore")
            semaphore.release()
            # ...and wait for it to become available again.
            utils.say("Waiting to acquire the semaphore")
            semaphore.acquire()

        s = utils.read_from_memory(mapfile)

    if what_i_wrote:
        try:
            assert(s == md5.new(what_i_wrote).hexdigest())
        except:
            raise AssertionError, "Shared memory corruption after %d iterations." % i

    what_i_wrote = md5.new(s).hexdigest()

    utils.write_to_memory(mapfile, what_i_wrote)
    
    if not params["LIVE_DANGEROUSLY"]:
        utils.say("Releasing the semaphore")
        semaphore.release()


utils.say("")
utils.say("%d iterations complete" % (i + 1))
